/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "mime-dialog.h"
#include "galeon-shell.h"
#include "eel-gconf-extensions.h"
#include "mime-db.h"
#include "gul-gui.h"

#include <gtk/gtkradiobutton.h>
#include <gtk/gtkentry.h>
#include <gtk/gtktogglebutton.h>
#include <libgnome/gnome-i18n.h>

struct MimeDialogPrivate
{
	MimeDB *mdb;
	MimeItem *selected_mime;
	
	GtkWidget *mime_list;
	GtkWidget *mime_action_frame;
	GtkWidget *mime_delete_button;
	GtkWidget *mime_type_entry;
	GtkWidget *mime_helper_frame;
	GtkWidget *mime_action;
	GtkWidget *mime_helper_entry;
	GtkWidget *mime_always_use;
	GtkWidget *mime_terminal;

	gboolean enable_edit_mime;
};

/* default MIME types */
extern struct
{
	gchar *name;
	MimeAction action;
} default_mimetypes[];

static void
mime_dialog_class_init (MimeDialogClass *klass);
static void
mime_dialog_init (MimeDialog *dialog);
static void
mime_dialog_finalize (GObject *object);


/* Callbacks */
void
prefs_mime_new_clicked_cb (GtkButton *button, MimeDialog *pd);
void
prefs_mime_delete_clicked_cb (GtkButton *button, MimeDialog *pd);
void
prefs_mime_type_changed_cb (GtkWidget *entry, MimeDialog *pd);
void
prefs_mime_action_clicked_cb (GtkWidget *button, MimeDialog *pd);
void
prefs_mime_helper_changed_cb (GtkWidget *entry, MimeDialog *pd);
void
prefs_mime_helper_buttons_toggled_cb (GtkWidget *button,
				      MimeDialog *pd);
void 
prefs_mime_close_button_clicked_cb (GtkWidget *button,
				    MimeDialog *pd);

static void
prefs_mime_selection_changed_cb (GtkTreeSelection *selection, MimeDialog *pd);
static void 
prefs_mime_list_unselected (MimeDialog *pd);

enum
{
        PROP_TREEVIEW,
        PROP_FILETYPE,
        PROP_ACTION,
	PROP_HELPER,
        PROP_TERMINAL,
	PROP_ALWAYS,
	PROP_ACTIONFRAME,
	PROP_DELETEBUTTON,
	PROP_HELPERFRAME
};

static const
GaleonDialogProperty properties [] =
{
        { PROP_TREEVIEW, "mime_list_treeview", NULL, PT_NORMAL, NULL },
	{ PROP_FILETYPE, "file_type_entry", NULL, PT_NORMAL, NULL },
	{ PROP_ACTION, "ask_user_radiobutton", NULL, PT_NORMAL, NULL },
	{ PROP_HELPER, "helper_entry", NULL, PT_NORMAL, NULL },
	{ PROP_TERMINAL, "terminal_checkbutton", NULL, PT_NORMAL, NULL },
	{ PROP_ALWAYS, "always_checkbutton", NULL, PT_NORMAL, NULL },
	{ PROP_ACTIONFRAME, "action_frame", NULL, PT_NORMAL, NULL },
	{ PROP_DELETEBUTTON, "delete_button", NULL, PT_NORMAL, NULL },
	{ PROP_HELPERFRAME, "helper_frame", NULL, PT_NORMAL, NULL },

	{ -1, NULL, NULL }
};

static GObjectClass *parent_class = NULL;

GType 
mime_dialog_get_type (void)
{
        static GType mime_dialog_type = 0;

        if (mime_dialog_type == 0)
        {
                static const GTypeInfo our_info =
                {
                        sizeof (MimeDialogClass),
                        NULL, /* base_init */
                        NULL, /* base_finalize */
                        (GClassInitFunc) mime_dialog_class_init,
                        NULL,
                        NULL, /* class_data */
                        sizeof (MimeDialog),
                        0, /* n_preallocs */
                        (GInstanceInitFunc) mime_dialog_init
                };

                mime_dialog_type = g_type_register_static (GALEON_DIALOG_TYPE,
                                                              "MimeDialog",
                                                              &our_info, 0);
        }

        return mime_dialog_type;

}

static void
mime_dialog_class_init (MimeDialogClass *klass)
{
        GObjectClass *object_class = G_OBJECT_CLASS (klass);

        parent_class = g_type_class_peek_parent (klass);

        object_class->finalize = mime_dialog_finalize;
}

GaleonDialog *
mime_dialog_new (GtkWidget *parent)
{
        return GALEON_DIALOG (g_object_new (MIME_DIALOG_TYPE,
                              "ParentWindow", parent,
                              NULL));
}

/**
 * prefs_mime_set_dialog_to_config: update the prefs dialog to reflect the
 * current mime setup
 */
static void
setup_treeview (MimeDialog *pd)
{
	gchar *text[4] = { NULL, NULL, NULL, NULL };
	GtkWidget *treeview = pd->priv->mime_list;
	const GList *l;
	GtkListStore *liststore;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkTreeSelection *selection;
	GtkTreeIter iter;
	GaleonEmbedShell *shell;
	const GList *mime_types;
	
	shell = galeon_shell_get_embed_shell (galeon_shell);
	pd->priv->mdb = galeon_embed_shell_get_mime_db (shell);

	liststore = gtk_list_store_new (5,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_POINTER);

	gtk_tree_view_set_model (GTK_TREE_VIEW(treeview), 
				 GTK_TREE_MODEL (liststore));
	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(treeview),
					   TRUE);

	renderer = gtk_cell_renderer_text_new ();

	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(treeview),
						     0, _("MIME Type"),
						     renderer,
						     "text", 0,
						     NULL);
	column = gtk_tree_view_get_column (GTK_TREE_VIEW(treeview), 0);
        gtk_tree_view_column_set_resizable (column, TRUE);
        gtk_tree_view_column_set_reorderable (column, TRUE);
        gtk_tree_view_column_set_sort_column_id (column, 0);

	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(treeview),
						     1, _("Action"),
						     renderer,
						     "text", 1,
						     NULL);
	column = gtk_tree_view_get_column (GTK_TREE_VIEW(treeview), 1);
        gtk_tree_view_column_set_resizable (column, TRUE);
        gtk_tree_view_column_set_reorderable (column, TRUE);
        gtk_tree_view_column_set_sort_column_id (column, 1);

	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(treeview),
						     2, _("Helper"),
						     renderer,
						     "text", 2,
						     NULL);
	column = gtk_tree_view_get_column (GTK_TREE_VIEW(treeview), 2);
        gtk_tree_view_column_set_resizable (column, TRUE);
        gtk_tree_view_column_set_reorderable (column, TRUE);
        gtk_tree_view_column_set_sort_column_id (column, 2);

	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(treeview),
						     3, _("Always Use"),
						     renderer,
						     "text", 3,
						     NULL);
	column = gtk_tree_view_get_column (GTK_TREE_VIEW(treeview), 3);
        gtk_tree_view_column_set_resizable (column, TRUE);
        gtk_tree_view_column_set_reorderable (column, TRUE);
        gtk_tree_view_column_set_sort_column_id (column, 3);

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(treeview));
        g_signal_connect (G_OBJECT (selection), "changed",
                          G_CALLBACK (prefs_mime_selection_changed_cb), pd);

	mime_types = mime_db_get_items (pd->priv->mdb);

	/* re-insert every element */
	for (l = mime_types; l != NULL; l = g_list_next(l))
	{
		MimeItem *m = (MimeItem *) (l->data);

		/* insert the row */
		text[0] = g_strdup (m->name);
		switch (m->action) {
		case MIME_SAVE_TO_DISK:
			text[1] = _("Save to disk");
			break;
		case MIME_RUN_PROGRAM:  
			text[1] = _("Run with Helper App");
			break;
		case MIME_UNKNOWN:
			text[1] = _("Unknown");
			break;
		case MIME_IGNORE:
			text[1] = _("Ignored");
			break;
		case MIME_ASK_ACTION:
			text[1] = _("Ask the user");
			break;
		}
		text[2] = g_strdup(m->helperApp);
		if (m->alwaysUse)
			text[3] = _("True");
		else
			text[3] = _("False");

		gtk_list_store_append (liststore, &iter);

		gtk_list_store_set (liststore,
				    &iter,
				    0, text[0],
				    1, text[1],
				    2, text[2],
				    3, text[3],
				    4, m,
				    -1);

		/* gtk copies these and its safe to free them right after they 
		   are appended */
		g_free (text[0]);
		g_free (text[2]);
	}
}

static void
mime_dialog_init (MimeDialog *dialog)
{       
        GaleonDialog *d = GALEON_DIALOG(dialog);
	
        dialog->priv = g_new0 (MimeDialogPrivate, 1);

	galeon_dialog_construct (GALEON_DIALOG(dialog),
                                 properties,
                                 "prefs-dialog.glade", 
                                 "mime_dialog");

	dialog->priv->mime_list = galeon_dialog_get_control (d, PROP_TREEVIEW);
	dialog->priv->mime_action_frame = galeon_dialog_get_control (d, PROP_ACTIONFRAME);
	dialog->priv->mime_delete_button = galeon_dialog_get_control (d, PROP_DELETEBUTTON);
	dialog->priv->mime_type_entry = galeon_dialog_get_control (d, PROP_FILETYPE);
	dialog->priv->mime_helper_frame = galeon_dialog_get_control (d, PROP_HELPERFRAME);
	dialog->priv->mime_action = galeon_dialog_get_control (d, PROP_ACTION);
	dialog->priv->mime_helper_entry = galeon_dialog_get_control (d, PROP_HELPER);
	dialog->priv->mime_always_use = galeon_dialog_get_control (d, PROP_ALWAYS);
	dialog->priv->mime_terminal = galeon_dialog_get_control (d, PROP_TERMINAL);
	
	setup_treeview (dialog);
}

static void
mime_dialog_finalize (GObject *object)
{
        MimeDialog *dialog;

        g_return_if_fail (object != NULL);
        g_return_if_fail (IS_MIME_DIALOG (object));

        dialog = MIME_DIALOG (object);

        g_return_if_fail (dialog->priv != NULL);
       
        g_free (dialog->priv);

        G_OBJECT_CLASS (parent_class)->finalize (object);
}

/* new mime type button clicked */
void
prefs_mime_new_clicked_cb (GtkButton *button, MimeDialog *pd)
{
	MimeItem *newitem;
	GtkWidget *treeview = pd->priv->mime_list;
	gchar *text[4];
	GtkListStore *liststore;
	GtkTreeSelection *selection;
	GtkTreeIter iter;

	newitem = mime_db_item_new (pd->priv->mdb,
                          	    _("New mime item"), 
                                    MIME_UNKNOWN,
                                    _("None"),
				    FALSE,
				    FALSE,
				    FALSE);

	text[0] = newitem->name;
	text[1] = _("Unknown");
	text[2] = newitem->helperApp;
	text[3] = _("False");

	liststore = GTK_LIST_STORE
		(gtk_tree_view_get_model 
		 (GTK_TREE_VIEW(treeview)));

	gtk_list_store_append (liststore, &iter);

	gtk_list_store_set (liststore,
			    &iter,
			    0, text[0],
			    1, text[1],
			    2, text[2],
			    3, text[3],
			    4, newitem,
			    -1);

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(treeview));
	gtk_tree_selection_select_iter (selection, &iter);
}

/* mime delete button clicked */
void
prefs_mime_delete_clicked_cb (GtkButton *button, MimeDialog *pd)
{
	GtkWidget *treeview = pd->priv->mime_list;
	GtkTreeModel *model;
	GtkTreeSelection *selection;
        GValue val = {0, };
	MimeItem *selected_mime;
	GtkTreeIter iter;
	GtkTreePath *path;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(treeview));
	g_return_if_fail (selection);
	gtk_tree_selection_get_selected (selection,
					 &model,
					 &iter);
        gtk_tree_model_get_value (model, &iter, 4, &val);
        selected_mime = (MimeItem *)g_value_get_pointer (&val);

	/* select next or previous item */
	path = gtk_tree_model_get_path (model, &iter);
	gtk_tree_path_next (path);
	gtk_tree_selection_select_path (selection, path);

	mime_db_item_remove (pd->priv->mdb, selected_mime);

	gtk_list_store_remove (GTK_LIST_STORE(model), &iter);

}

static void
prefs_mime_selection_changed_cb (GtkTreeSelection *selection, MimeDialog *pd)
{
	gint act_num = 0;
	GtkTreeIter iter;
	GValue val = {0, };
	MimeItem *selected_mime;
	GtkTreeModel *model;

	pd->priv->enable_edit_mime = FALSE;

	if (!gtk_tree_selection_get_selected (selection,
					      &model,
					      &iter))
	{
		prefs_mime_list_unselected (pd);
		return;
	}

	gtk_tree_model_get_value (model, &iter, 4, &val);
        selected_mime = (MimeItem *)g_value_get_pointer (&val);

	switch (selected_mime->action)
	{
	case MIME_SAVE_TO_DISK:
		act_num = 0;	/* save */
		break;
	case MIME_RUN_PROGRAM:	/* run with helper app */
		act_num = 1;
		break;
	case MIME_ASK_ACTION:	/* Ask */
	case MIME_UNKNOWN:	/* default to ask */
		act_num = 2;
		break;
	case MIME_IGNORE:
		break;
	}

	gtk_widget_set_sensitive (pd->priv->mime_action_frame, TRUE);
	gtk_widget_set_sensitive (pd->priv->mime_delete_button, TRUE);
	gtk_widget_set_sensitive (pd->priv->mime_type_entry, TRUE);
	gtk_widget_set_sensitive (pd->priv->mime_helper_frame, act_num);

	gul_gui_gtk_radio_button_set (GTK_RADIO_BUTTON (pd->priv->mime_action),
			      	      act_num);
	gtk_entry_set_text (GTK_ENTRY (pd->priv->mime_type_entry), 
			    selected_mime->name);
	gtk_entry_set_text (GTK_ENTRY (pd->priv->mime_helper_entry),
			    selected_mime->helperApp);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pd->priv->mime_always_use),
				      selected_mime->alwaysUse);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pd->priv->mime_terminal),
				      selected_mime->terminalHelper);

	pd->priv->enable_edit_mime = TRUE;
}

static void 
prefs_mime_list_unselected (MimeDialog *pd)
{
	pd->priv->selected_mime = NULL;
	pd->priv->enable_edit_mime = FALSE;

	/* The default is Ask */
	gul_gui_gtk_radio_button_set (GTK_RADIO_BUTTON (pd->priv->mime_action), 2);
	gtk_entry_set_text (GTK_ENTRY (pd->priv->mime_type_entry), "");
	gtk_entry_set_text (GTK_ENTRY (pd->priv->mime_helper_entry), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pd->priv->mime_always_use),
				      FALSE);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pd->priv->mime_terminal),
				      FALSE);
	gtk_widget_set_sensitive (pd->priv->mime_action_frame, FALSE);
	gtk_widget_set_sensitive (pd->priv->mime_helper_frame, FALSE);
	gtk_widget_set_sensitive (pd->priv->mime_type_entry, FALSE);
	gtk_widget_set_sensitive (pd->priv->mime_delete_button, FALSE);
}

/* handles changes in the mime type entry */
void
prefs_mime_type_changed_cb (GtkWidget *entry, MimeDialog *pd)
{
	GtkWidget *treeview = pd->priv->mime_list;
	GtkTreeModel *model;
	GtkTreeSelection *selection;
        GValue val = {0, };
	MimeItem *selected_mime;
	GtkTreeIter iter;
	gboolean res;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(treeview));
	res = gtk_tree_selection_get_selected (selection,
					       &model,
					       &iter);
	if (!res) return;
	gtk_tree_model_get_value (model, &iter, 4, &val);
        selected_mime = (MimeItem *)g_value_get_pointer (&val);

	if (selected_mime == NULL || !pd->priv->enable_edit_mime)
		return;

	if (selected_mime->name)
		g_free (selected_mime->name);
	selected_mime->name = gtk_editable_get_chars 
		(GTK_EDITABLE (pd->priv->mime_type_entry), 0, -1);

	gtk_list_store_set (GTK_LIST_STORE(model),
			    &iter,
			    0, selected_mime->name,
			    -1);
}

/* handles changes in the mime action */
void
prefs_mime_action_clicked_cb (GtkWidget *button, MimeDialog *pd)
{
	gint state;
	GtkWidget *treeview = pd->priv->mime_list;
	gchar *name;
	GtkTreeModel *model;
	GtkTreeSelection *selection;
        GValue val = {0, };
	MimeItem *selected_mime;
	GtkTreeIter iter;
	gboolean res;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(treeview));
	res = gtk_tree_selection_get_selected (selection,
					       &model,
					       &iter);
	if (!res) return;
	gtk_tree_model_get_value (model, &iter, 4, &val);
        selected_mime = (MimeItem *)g_value_get_pointer (&val);


	if (!pd->priv->enable_edit_mime)
		return;

	state = gul_gui_gtk_radio_button_get (GTK_RADIO_BUTTON (
					      pd->priv->mime_action));
	switch (state)
	{
		case 0:
			selected_mime->action = MIME_SAVE_TO_DISK;
			name = _("Save to disk");
			break;
		case 1:
			selected_mime->action = MIME_RUN_PROGRAM;
			name = _("Run with Helper App");
			break;
		case 2:
			selected_mime->action = MIME_ASK_ACTION;
			name = _("Ask the user");
			break;
		default:
			selected_mime->action = MIME_UNKNOWN;
			name = _("Unknown");
			break;
	}

	gtk_list_store_set (GTK_LIST_STORE(model),
			    &iter,
			    1, name,
			    -1);

	/* set the helper frame sensitive if the type is run or ask */
	gtk_widget_set_sensitive (pd->priv->mime_helper_frame, state);
}

/* handles changes in the helper app entry */
void
prefs_mime_helper_changed_cb (GtkWidget *entry, MimeDialog *pd)
{
	GtkWidget *treeview = pd->priv->mime_list;
	GtkTreeSelection *selection;
        GValue val = {0, };
	MimeItem *selected_mime;
	GtkTreeModel *model;
	GtkTreeIter iter;
	gboolean res;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(treeview));
	res = gtk_tree_selection_get_selected (selection,
					       &model,
					       &iter);
	if (!res) return;
	
	gtk_tree_model_get_value (model, &iter, 4, &val);
	
        selected_mime = (MimeItem *)g_value_get_pointer (&val);
	if (selected_mime == NULL || !pd->priv->enable_edit_mime) return;

	g_free (selected_mime->helperApp);
	selected_mime->helperApp = gtk_editable_get_chars 
		(GTK_EDITABLE (pd->priv->mime_helper_entry), 0, -1);

	/* update the clist */
	gtk_list_store_set (GTK_LIST_STORE(model),
			    &iter,
			    2, selected_mime->helperApp,
			    -1);
}

/* handles changes in the mime helper buttons' states */
void
prefs_mime_helper_buttons_toggled_cb (GtkWidget *button,
				      MimeDialog *pd)
{
	gchar *text;
	GtkWidget *treeview = pd->priv->mime_list;
	GtkTreeSelection *selection;
        GValue val = {0, };
	MimeItem *selected_mime;
	GtkTreeModel *model;
	GtkTreeIter iter;
	gboolean res;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(treeview));
	res = gtk_tree_selection_get_selected (selection,
					       &model,
					       &iter);
	if (!res) return;
	gtk_tree_model_get_value (model, &iter, 4, &val);
        selected_mime = (MimeItem *)g_value_get_pointer (&val);

	if (!pd->priv->enable_edit_mime)
		return;

	selected_mime->terminalHelper = gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON(pd->priv->mime_terminal));
	selected_mime->alwaysUse = gtk_toggle_button_get_active
		(GTK_TOGGLE_BUTTON(pd->priv->mime_always_use));

	if (selected_mime->alwaysUse)
		text = _("True");
	else
		text = _("False");

	gtk_list_store_set (GTK_LIST_STORE(model),
			    &iter,
			    3, text,
			    -1);
}

void 
prefs_mime_close_button_clicked_cb (GtkWidget *button,
				    MimeDialog *pd)
{
	g_object_unref (pd);
}
